#!/bin/bash

cfgfile_gateway=/etc/router/config/lan_gateway.json
cfgfile_dhcp=/etc/router/config/lan_dhcp.json

start_dnsmasq() {
	gateway=`cat ${cfgfile_gateway} | jq -r .ipaddr`
	netmask=`cat ${cfgfile_gateway} | jq -r .netmask`
	start=`cat ${cfgfile_dhcp} | jq -r .start`
	end=`cat ${cfgfile_dhcp} | jq -r .end`
	leasetime=`cat ${cfgfile_dhcp} | jq -r .leasetime`
	dns_manual=`cat ${cfgfile_dhcp} | jq -r .dns_manual`
	dns1=`cat ${cfgfile_dhcp} | jq -r .dns1`
	dns2=`cat ${cfgfile_dhcp} | jq -r .dns2`

	dnsmasq_cfg_path=/etc/router/run/dnsmasq.conf
	dnsmasq_pid_path=/etc/router/run/dnsmasq.pid
	dnsmasq_leases_path=/etc/router/run/dnsmasq.leases

	proc=`ps -ef | grep ${dnsmasq_cfg_path} | grep -v grep | awk '{print $2}'`
	if [[ -n ${proc} ]]; then
		kill -9 ${proc}
		sleep 0.5
	fi
	
	cat > ${dnsmasq_cfg_path} << EOF
bind-interfaces
listen-address=${gateway}
dhcp-range=${start},${end},${netmask},${leasetime}h
dhcp-option-force=option:router,${gateway}
dhcp-option-force=option:mtu,1492
EOF

	if [[ ${dns_manual} == 1 ]]; then
		echo "dhcp-option-force=option:dns-server,${dns1},${dns2}" >> ${dnsmasq_cfg_path}
	fi

	ifconfig br0 ${gateway} netmask ${netmask} up
	dnsmasq -C ${dnsmasq_cfg_path} -x ${dnsmasq_pid_path} -l ${dnsmasq_leases_path}
}

start_dnsmasq