#!/bin/bash

cfgfile_5g=/etc/router/config/wireless_5g.json

start_wireless_5g() {
	local ifname=$1
	enabled=`cat ${cfgfile_5g} | jq -r .enabled`
	ssid=`cat ${cfgfile_5g} | jq -r .ssid`
	encryption=`cat ${cfgfile_5g} | jq -r .encryption`
	key=`cat ${cfgfile_5g} | jq -r .key`
	country=`cat ${cfgfile_5g} | jq -r .country`
	if [[ ${country} == null ]]; then
		country=US
	fi
	txpower=`cat ${cfgfile_5g} | jq -r .txpower`
	if [[ ${txpower} == null ]]; then
		txpower=20
	fi

	hostapd_cfgpath=/etc/router/run/hostapd.conf-${ifname}
	proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
	if [[ -n ${proc} ]]; then
		kill -9 ${proc}
		sleep 0.5
	fi

	if [[ ${enabled} == 0 ]]; then
		echo "wireless 5g disabled!"
		return
	fi
	
	iwconfig ${ifname} txpower ${txpower}

	#driver_type=`ethtool -i ${ifname} | grep driver`
	bus_id=`ls -l /sys/class/net/${ifname}/device | awk -F'0000:' '{print $2}'`
	netcard_model=`lspci -s ${bus_id} | awk -F': ' '{print $2}'`
	if [[ ${netcard_model} =~ "QCA9984" ]]; then
	cat > ${hostapd_cfgpath} << EOF
driver=nl80211
hw_mode=a
wmm_enabled=1
bridge=br0
ieee80211n=1
ht_capab=[HT40+][LDPC][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][MAX-AMSDU-7935][DSSS_CCK-40]
ieee80211ac=1
vht_capab=[RXLDPC][SHORT-GI-80][SHORT-GI-160][TX-STBC-2BY1][SU-BEAMFORMER][SU-BEAMFORMEE][MU-BEAMFORMEE][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN][RX-STBC-1][VHT160-80PLUS80][MAX-MPDU-11454][MAX-A-MPDU-LEN-EXP7]
vht_oper_chwidth=1
channel=${channel}
acs_num_scans=5
country_code=${country}
ctrl_interface=/var/run/hostapd
auth_algs=1
wpa_key_mgmt=WPA-PSK
wpa_pairwise=CCMP
wpa=${encryption}
interface=${ifname}
ssid=${ssid}
wpa_passphrase=${key}
EOF
	elif [[ ${netcard_model} =~ "7615" ]]; then
	cat > ${hostapd_cfgpath} << EOF
driver=nl80211
hw_mode=a
wmm_enabled=1
bridge=br0
ieee80211n=1
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
ieee80211ac=1
vht_capab=[RXLDPC][TX-STBC-2BY1][SHORT-GI-80][MAX-A-MPDU-LEN-EXP7]
vht_oper_chwidth=1
channel=${channel}
acs_num_scans=1
country_code=${country}
ctrl_interface=/var/run/hostapd
auth_algs=1
wpa_key_mgmt=WPA-PSK
wpa_pairwise=CCMP
wpa=${encryption}
interface=${ifname}
ssid=${ssid}
wpa_passphrase=${key}
EOF
	else
	cat > ${hostapd_cfgpath} << EOF
driver=nl80211
hw_mode=a
wmm_enabled=1
bridge=br0
ieee80211n=1
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
ieee80211ac=1
vht_capab=[RXLDPC]
vht_oper_chwidth=0
channel=${channel}
acs_num_scans=1
country_code=${country}
ctrl_interface=/var/run/hostapd
auth_algs=1
wpa_key_mgmt=WPA-PSK
wpa_pairwise=CCMP
wpa=${encryption}
interface=${ifname}
ssid=${ssid}
wpa_passphrase=${key}
EOF
	fi
	
	for i in {1..3}
	do
		hostapd -B ${hostapd_cfgpath}
		sleep 2
		proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
		if [[ -n ${proc} ]]; then
			return 0
		fi
	done
	exit 1
}

check_wireless_channel() {
	local netdev=$1
	wireless_dir="/sys/class/net/"${netdev}"/wireless"
	if [[ -d ${wireless_dir} ]]; then
		support_5g=`iwlist ${netdev} channel | grep "Channel 36"`
		if [[ -n ${support_5g} ]]; then
			start_wireless_5g ${netdev}
		fi
	fi
}

scan_wireless_dev() {
	netdevs=`ls /sys/class/net/ | grep -v "$(ls /sys/devices/virtual/net/)"`
	for netdev in ${netdevs[@]}
	do
		check_wireless_channel ${netdev}
	done
}

cfgfile_wan_method=/etc/router/config/wan_method.json
setup=`cat ${cfgfile_wan_method} | jq -r .setup`
channel=0

if [[ ${setup} == 7 ]]; then
	#cfgfile_wan=/etc/router/config/wan_wireless_repeater.json
	#channel=`cat ${cfgfile_wan} | jq -r .channel`
	netdev=wlan0-ap
	check_wireless_channel ${netdev}
else
	scan_wireless_dev
fi
