#!/bin/bash

cfgfile=/etc/router/config/storage_create_pool.json
datadir=

get_unused_datadir() {
	for i in $(seq 0 99)
	do
		datadir="/data"$i
		mount_points=`df | awk '{print $6}' | grep "^${datadir}"`
		if [[ -z ${mount_points} ]]; then
			mkdir -p ${datadir}
			chmod 777 ${datadir}
			return
		fi
	done
	echo "Could not find unused datadir..." >&2
	exit 1
}

dev_mount() {
	local dev=$1
	
	# mount partition
	get_unused_datadir
	systemctl daemon-reload
	local uuid=`lsblk -no uuid ${dev} | head -1`
	local fstype=`lsblk -no fstype ${dev} | head -1`
	sed -i "\#  ${datadir}  #d" /etc/fstab
	echo "UUID=${uuid}  ${datadir}  ${fstype}  defaults  0  0 " >>/etc/fstab
	sleep 0.5
	mount -a
}

raid_level=`cat ${cfgfile} | jq -r .raid_level`
disk_num=`cat ${cfgfile} | jq -r .drive_num`
disk_list=
raid_dev=
for i in $(seq 0 $[$disk_num-1])
do
	disk=`cat ${cfgfile} | jq -r .drives[$i]`
	disk_list=${disk_list}" "${disk}1
done

# get an unused raid device name
for i in $(seq 0 99)
do
	tmp_dev="/dev/md"$i
	if [[ ! -b ${tmp_dev} ]]; then
		raid_dev=${tmp_dev}
		break
	fi
done
if [[ -z ${raid_dev} ]]; then
	echo "Could not find unused raiddev..." >&2
	exit 1
fi

# clear disk part
for i in $(seq 0 $[$disk_num-1])
do
	disk=`cat ${cfgfile} | jq -r .drives[$i]`
	# parted disk
	parted -s ${disk} mklabel gpt mkpart primary 0% 100%
	sleep 1
	# clear disk superblock info
	diskdev=${disk}1
	old_type=`lsblk -no fstype ${diskdev} | head -1`
	if [[ ${old_type} == linux_raid_member ]]; then
		mddev=`cat /proc/mdstat | grep ${diskdev:5} | awk '{print $1}'`
		if [[ -n ${mddev} ]]; then
			mdadm -S /dev/${mddev}
			mdadm --zero-superblock ${diskdev}
			sleep 1
		fi
		old_type=`lsblk -no fstype ${diskdev} | head -1`
		if [[ ${old_type} == linux_raid_member ]]; then
			echo "data disk clean superblock fail..." >&2
			exit 1
		fi
		parted -s ${disk} mklabel gpt mkpart primary 0% 100%
		sleep 1
	fi
	parted -s ${disk} set 1 raid on
done
sleep 0.5

# create new raid
mdadm -C ${raid_dev} -l ${raid_level} -n ${disk_num} ${disk_list} << EOF
y
EOF
sleep 0.5

partprobe
mkfs.xfs -f ${raid_dev}
sleep 1
dev_mount ${raid_dev}

# check mount result
mount_dev=`df | awk '{print $1}' | grep "^${raid_dev}"`
if [[ -z ${mount_dev} ]]; then
	echo "${raid_dev} is not mounted, it will be deleted..." >&2
	sed -i "\#  ${datadir}  #d" /etc/fstab
	# stop raid device
	mdadm -D ${raid_dev}
	if [[ $? == 0 ]]; then
		mdadm -S ${raid_dev}
	fi
	exit 1
fi

exit 0
