#!/bin/bash

cfgfile=/etc/router/config/storage_create_volume.json
datadir=

get_unused_datadir() {
	for i in $(seq 0 99)
	do
		datadir="/data"$i
		mount_points=`df | awk '{print $6}' | grep "^${datadir}"`
		if [[ -z ${mount_points} ]]; then
			mkdir -p ${datadir}
			chmod 777 ${datadir}
			return
		fi
	done
	echo "Could not find unused datadir..." >&2
	exit 1
}

dev_formatting() {
	local dev=$1
	local fstype=$2

	# formatting partition
	if [[ ${fstype} == xfs ]]; then
		# exfat can clear superblock info
		mkfs.exfat ${dev}
		mkfs.xfs -f ${dev}
	elif [[ ${fstype} == exfat ]]; then
		mkfs.exfat ${dev}
	else
		echo "data disk fstype unknown..." >&2
		exit 1
	fi
	sleep 0.5
}

dev_mount() {
	local dev=$1
	local fstype=$2
	
	# mount partition
	get_unused_datadir
	local uuid=`lsblk -no uuid ${dev} | head -1`
	#local fstype=`lsblk -no fstype ${dev} | head -1`
	sed -i "\#  ${datadir}  #d" /etc/fstab
	echo "UUID=${uuid}  ${datadir}  ${fstype}  defaults  0  0 " >>/etc/fstab
	sleep 0.5
	systemctl daemon-reload
	mount -a
}

disk_single() {
	disk=`cat ${cfgfile} | jq -r .path`
	fstype=`cat ${cfgfile} | jq -r .fstype`
	diskdev=${disk}1

	# parted disk
	parted -s ${disk} mklabel gpt mkpart primary 0% 100%
	sleep 1
	# clear disk raid info
	old_type=`lsblk -no fstype ${diskdev} | head -1`
	if [[ ${old_type} == linux_raid_member ]]; then
		mddev=`cat /proc/mdstat | grep ${diskdev:5} | awk '{print $1}'`
		if [[ -n ${mddev} ]]; then
			mdadm -S ${mddev}
			mdadm --zero-superblock ${diskdev}
		fi
		old_type=`lsblk -no fstype ${diskdev} | head -1`
		if [[ ${old_type} == linux_raid_member ]]; then
			echo "data disk clean superblock fail..." >&2
			exit 1
		fi
		parted -s ${disk} mklabel gpt mkpart primary 0% 100%
		sleep 1
	fi

	partprobe
	sleep 0.5
	dev_formatting ${diskdev} ${fstype}
	dev_mount ${diskdev} ${fstype}

	# check mount result
	mount_dev=`df | awk '{print $1}' | grep "^${diskdev}"`
	if [[ -z ${mount_dev} ]]; then
		echo "${diskdev} is not mounted, it will be deleted..." >&2
		sed -i "\#  ${datadir}  #d" /etc/fstab
		exit 1
	fi

	exit 0
}

disk_raid() {
	exit 0
}

type=`cat ${cfgfile} | jq -r .type`
case "${type}" in
  0)
    disk_single
    ;;
  1)
    disk_raid
    ;;
  *)
    exit 1
esac
