local router = {}
function router.process_error(code, info)
	local cjson = require("cjson")
	local temp = {}
	temp["err_code"] = code
	temp["err_info"] = info
	local resp = cjson.encode(temp)
	if (code ~= 0) then
		ngx.log(ngx.ERR, info)
	end
	ngx.say(resp)
	return 0
end

function router.set_wan_method(mode)
	local cjson = require("cjson")
	local temp = {}
	temp["setup"] = mode
	local json_data = cjson.encode(temp)
	local file = io.open("/etc/router/config/wan_method.json", "w")
	if (file == nil) then
		router.process_error(1, "set_wan_method: open file failed!")
		return -1
	end
	file:write(json_data)
	file:close()
	return 0
end

function router.read_config(file_name)
	local file = io.open(file_name, "r");
	if (file == nil) then
		router.process_error(1, "read_config: open file failed!")
		return -1
	end
	local data = file:read("*a")
	file:close()
	ngx.say(data)
	return 0
end

function router.write_config(file_name, op_cmd)
	local body_data = ngx.req.get_body_data()
	local file = io.open(file_name, "w");
	if (file == nil) then
		router.process_error(2, "write_config: open file failed!")
		return -1
	end
	file:write(body_data)
	file:close()

	local os_ret = os.execute(op_cmd)
	if (os_ret == nil) then
		router.process_error(3, "server operation failed!")
		return -1
	end
	router.process_error(0, "ok")
	return 0
end


return router
