local router = require("router")
local cjson = require("cjson")

function get_init_network()
	local file_handle = io.popen('ls /sys/class/net/ | grep -v "$(ls /sys/devices/virtual/net/)"')
	local net_list = {}
	setmetatable(net_list, cjson.empty_array_mt)
	local net_5g_list = {}
	setmetatable(net_5g_list, cjson.empty_array_mt)
	for ifname in file_handle:lines()
	do
		local cmd = "test -d /sys/class/net/"..ifname.."/wireless"
		local os_ret = os.execute(cmd)
		if (os_ret == true) then
			local mac_file = io.popen("cat /sys/class/net/"..ifname.."/address | awk -F: '{print $5 $6}' | tr -d '\n'")
			local mac = mac_file:read("*a")
			mac_file:close()
			local info = {}
			info["ifname"] = ifname
			info["ssid"] = "PearBox-"..mac
			local ret = os.execute("iwlist "..ifname.." channel | grep 'Channel 36'")
			if (ret == nil) then
				table.insert(net_list, info)
			else
				table.insert(net_5g_list, info)
			end
		end
	end
	file_handle:close()

	local temp = {}
	local os_ret = os.execute("ip addr show dev eth0 | grep dynamic")
	if (os_ret == nil) then
		temp["setup"] = 1
	else
		temp["setup"] = 4
	end
	temp["wireless_2g"] = net_list
	temp["wireless_5g"] = net_5g_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

function post_init_user()
	os.execute("rm /usr/local/openresty/nginx/kodbox/data/system/install.lock")
	local os_ret = os.execute("test -f /usr/local/openresty/nginx/kodbox/data/system/install.lock")
	if (os_ret == true) then
		router.process_error(3, "install.lock can't delete!")
		return -1
	end

	local body_data = ngx.req.get_body_data()
	local req = cjson.decode(body_data)
	local res = ngx.location.capture("/koduser",{
		method = ngx.HTTP_POST,
		args = {name=req["username"], password=req["password"]}
	});
	if (200 ~= res.status) then
		router.process_error(1, "parameter is invalid!")
		return -1
	end
	local res_body_data = cjson.decode(res.body)
	if (res_body_data["code"] == false) then
		router.process_error(1, "user name or password is incorrect!")
		return -1
	end

	router.write_config("/etc/router/config/user_info.json", "echo 'init user success!'")

end

function modify_storage()
	local file = io.open("/etc/router/config/user_info.json", "r");
	if (file == nil) then
		router.process_error(1, "user_login: open file failed!")
		return
	end
	local file_data = file:read("*a");
	file:close()

	local user = cjson.decode(file_data)
	local res = ngx.location.capture("/kodlogin", {
		method = ngx.HTTP_GET,
		args = {name=user["username"], password=user["password"]}
	});
	if (200 ~= res.status) then
		router.process_error(1, "user name or password is incorrect!")
		return -1
	end
	local res_body_data = cjson.decode(res.body)
	if (res_body_data["code"] == false) then
		router.process_error(1, "user name or password is incorrect!")
		return -1
	end

	local token = res_body_data["info"]
	local body_data = ngx.req.get_body_data()
	local req = cjson.decode(body_data)

	res = ngx.location.capture("/kodstorage",{
		method = ngx.HTTP_POST,
		args = {accessToken=token}
	});
	if (200 ~= res.status) then
		router.process_error(1, "Route-Token is invalid!")
		return -1
	end
	res_body_data = cjson.decode(res.body)
	if (res_body_data["code"] ~= true) then
		router.process_error(1, "user name or password is incorrect!")
		return -1
	end
end

function post_init_finish()
	local os_arg = "not_move_sql"
	local os_ret = os.execute("df | awk '{print $6}' | grep '^/data0'")
	if (os_ret == true) then
		modify_storage()
		--os_arg = "move_sql"
	end

	router.process_error(0, "ok")
	local os_ret = os.execute("/etc/router/scripts/init_finish.sh "..os_arg)
	if (os_ret == nil) then
		router.process_error(3, "server operation failed!")
		return -1
	end
end

if (ngx.var.uri == '/init/network') then
	if (ngx.var.request_method == 'GET') then
		get_init_network()
	end
elseif (ngx.var.uri == '/init/user') then
	if (ngx.var.request_method == 'POST') then
		post_init_user()
	end
elseif (ngx.var.uri == '/init/finish') then
	if (ngx.var.request_method == 'POST') then
		post_init_finish()
	end
else
	router.process_error(1, "invalid uri: "..ngx.var.uri)
end
