local router = require("router")
local cjson = require("cjson")

function get_network_status()
	local interface_list = {}
	setmetatable(interface_list, cjson.empty_array_mt)

	local file = io.open("/etc/router/config/wan_method.json", "r")
	if (file == nil) then
		router.process_error(1, "read_config: open file failed!")
		return -1
	end
	local file_data = file:read("*a")
	file:close()
	local wan_method = cjson.decode(file_data)
	local setup = wan_method["setup"]

	if (setup == 2 or setup == 3) then
		local os_ret = os.execute("cat /etc/router/run/account_list.txt")
		if (os_ret == nil) then
			router.process_error(3, "account_list.txt does not exist!")
			return -1
		end

		local seq = 0
		local pf = io.popen("cat /etc/router/run/account_list.txt | awk '{print $1}'")
		for user in pf:lines()
		do
			local interface = {}
			interface["username"] = user
			local ifname = "ppp"..seq
			seq = seq + 1
			local status = os.execute("ifconfig "..ifname)
			if (status == true) then
				local pf2 = io.popen("ifconfig "..ifname.." | grep inet |  awk '{print $2}'")
				local ipaddr = pf2:read()
				pf2:close()
				if (ipaddr ~= nil) then
					pf2 = io.popen("ifconfig "..ifname.." | grep 'RX packets' | awk -F'(' '{print $2}' | awk -F')' '{print $1}'")
					local rx = pf2:read()
					pf2:close()
					pf2 = io.popen("ifconfig "..ifname.." | grep 'TX packets' | awk -F'(' '{print $2}' | awk -F')' '{print $1}'")
					local tx = pf2:read()
					pf2:close()

					interface["ifname"] = ifname
					interface["ipv4"] = ipaddr
					interface["rx"] = rx
					interface["tx"] = tx
				end
			end
			table.insert(interface_list, interface)
		end
		pf:close()
	else
		local pf = io.popen('ls /sys/class/net/ | grep -v "$(ls /sys/devices/virtual/net/ | grep -v ppp)"')
		for ifname in pf:lines()
		do
			local pf2 = io.popen("ifconfig "..ifname.." | grep inet |  awk '{print $2}'")
			local ipaddr = pf2:read()
			pf2:close()
			if (ipaddr ~= nil) then
				pf2 = io.popen("ifconfig "..ifname.." | grep 'RX packets' | awk -F'(' '{print $2}' | awk -F')' '{print $1}'")
				local rx = pf2:read()
				pf2:close()
				pf2 = io.popen("ifconfig "..ifname.." | grep 'TX packets' | awk -F'(' '{print $2}' | awk -F')' '{print $1}'")
				local tx = pf2:read()
				pf2:close()

				local interface = {}
				interface["ifname"] = ifname
				interface["ipv4"] = ipaddr
				interface["rx"] = rx
				interface["tx"] = tx
				table.insert(interface_list, interface)
			end
		end
		pf:close()
	end

	local temp = {}
	temp["setup"] = setup
	local status = os.execute("curl cip.cc")
	temp["status"] = status and "connected" or "disconnected"
	temp["interfaces"] = interface_list
	local resp = cjson.encode(temp)
	ngx.say(resp)

end

function scan_wireless_hotspots()
	local os_ret = os.execute("/etc/router/scripts/scan_wireless_hotspots.sh")
	if (os_ret == nil) then
		router.process_error(3, "server operation failed!")
		return -1
	end

	local pf = io.popen("cat /etc/router/run/scan_result.txt | grep ESSID  | awk -F: '{print $2}' | sed 's/\"//g'")
	local index = 0
	local ssid = {}
	for str in pf:lines() do
		index = index + 1
		ssid[index] = str
	end
	pf:close()
	local num = index

	pf = io.popen("cat /etc/router/run/scan_result.txt | grep 'Encryption key' | awk -F: '{print $2}'")
	index = 0
	local encryption = {}
	for str in pf:lines() do
		index = index + 1
		encryption[index] = str
	end
	pf:close()

	pf = io.popen("cat /etc/router/run/scan_result.txt | grep 'Signal level'|awk -F= '{print $3}' | awk  '{print $1}'")
	index = 0
	local signal = {}
	for str in pf:lines() do
		index = index + 1
		quality = (tonumber(str) + 100) * 2
		if (quality > 100) then
			quality = 100
		end
		if (quality < 0) then
			quality = 0
		end
		signal[index] = quality
	end
	pf:close()

	pf = io.popen("cat /etc/router/run/scan_result.txt | grep Channel: | awk -F: '{print $2}'")
	index = 0
	local channel = {}
	for str in pf:lines() do
		index = index + 1
		channel[index] = tonumber(str)
	end
	pf:close()

	local hotspot_list = {}
	setmetatable(hotspot_list, cjson.empty_array_mt)
	for i = 1, num do
		local hotspot = {}
		hotspot["ssid"] = ssid[i]
		hotspot["encryption"] = encryption[i]
		hotspot["quality"] = signal[i]
		hotspot["channel"] = channel[i]
		table.insert(hotspot_list, hotspot)
	end

	local temp = {}
	temp["result"] = hotspot_list
	local resp = cjson.encode(temp)
	ngx.say(resp)
end

if (ngx.var.uri == '/network/wan/pppoe/dial') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_pppoe_dial.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(1)
		router.write_config("/etc/router/config/wan_pppoe_dial.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/pppoe/multidial') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_pppoe_multidial.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(2)
		router.write_config("/etc/router/config/wan_pppoe_multidial.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/pppoe/multiwan') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_pppoe_multiwan.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(3)
		router.write_config("/etc/router/config/wan_pppoe_multiwan.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/dhcp') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_dhcp.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(4)
		router.write_config("/etc/router/config/wan_dhcp.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/static') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_static.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(5)
		router.write_config("/etc/router/config/wan_static.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/bridge') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_bridge.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(6)
		router.write_config("/etc/router/config/wan_bridge.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/wireless/repeater') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_wireless_repeater.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.set_wan_method(7)
		router.write_config("/etc/router/config/wan_wireless_repeater.json", "/etc/router/scripts/start_wan.sh")
	end
elseif (ngx.var.uri == '/network/wan/method') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wan_method.json")
	end
elseif (ngx.var.uri == '/network/wireless/2g') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wireless_2g.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.write_config("/etc/router/config/wireless_2g.json", "/etc/router/scripts/start_wireless_2g.sh")
	end
elseif (ngx.var.uri == '/network/wireless/5g') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/wireless_5g.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.write_config("/etc/router/config/wireless_5g.json", "/etc/router/scripts/start_wireless_5g.sh")
	end
elseif (ngx.var.uri == '/network/wireless/hotspots') then
	if (ngx.var.request_method == 'GET') then
		scan_wireless_hotspots()
	end
elseif (ngx.var.uri == '/network/lan/gateway') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/lan_gateway.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.write_config("/etc/router/config/lan_gateway.json", "/etc/router/scripts/start_lan.sh")
	end
elseif (ngx.var.uri == '/network/lan/dhcp') then
	if (ngx.var.request_method == 'GET') then
		router.read_config("/etc/router/config/lan_dhcp.json")
	elseif (ngx.var.request_method == 'PUT') then
		router.write_config("/etc/router/config/lan_dhcp.json", "/etc/router/scripts/start_lan.sh")
	end
elseif (ngx.var.uri == '/network/lan/devices') then
	if (ngx.var.request_method == 'GET') then
		local cjson = require("cjson")
		local devices = {}
		setmetatable(devices, cjson.empty_array_mt)
		local file = io.open("/etc/router/run/dnsmasq.leases", "r");
		for str in file:lines() do
			local macaddr, ipaddr, devname = string.match(str, "^[%d]+%s([%x:]+)%s([%d.]+)%s([^%s]+)")
			local device = {}
			device["devname"] = devname
			device["ipaddr"] = ipaddr
			device["macaddr"] = macaddr
			local online = os.execute("ping -c 1 -W 0.01 "..ipaddr)
			device["online"] = online and 1 or 0
			table.insert(devices, device)
		end
		file:close()
		local resp = cjson.encode(devices)
		ngx.say(resp)
	end
elseif (ngx.var.uri == '/network/wan/status') then
	if (ngx.var.request_method == 'GET') then
		get_network_status()
	end
else
	router.process_error(1, "invalid uri: "..ngx.var.uri)
end
